import logging
from typing import Tuple

from vimania_todos.buku import BukuDb
from vimania_todos.environment import config
from vimania_todos.exception import VimaniaException
from vimania_todos.pattern import URL_PATTERN

_log = logging.getLogger("vimania-todos.bms")


def add_twbm(url: str) -> int:
    id_ = BukuDb(dbfile=config.dbfile_twbm).add_rec(
        url=url,
        # title_in=title,
        tags_in=",vimania,",
        # desc=desc,
        # immutable=0,
        delay_commit=False,
        # fetch=(not nofetch),
    )
    if id_ == -1:
        # raise SystemError(f"Error adding {url=} to DB {config.dbfile_twbm}")
        _log.error(
            f"Error adding {url=} to DB {config.dbfile_twbm}"
        )  # TODO: buku.py error handling
    else:
        _log.debug(f"Added twbm: {id_=} - {url} to DB {config.dbfile_twbm}")
    return id_


def delete_twbm(line: str) -> Tuple[int, str]:
    """Delete bookmarks, managed by vimania (tag: vimania)"""
    match = URL_PATTERN.match(line)
    if match is None:
        _log.warning(f"Cannot extract url from: {line}")
        raise VimaniaException(f"Cannot extract url from: {line}")

    url = match.group(1)
    id_ = BukuDb(dbfile=config.dbfile_twbm).get_rec_id(url=url)  # exact match
    if id_ == -1:
        _log.info(f"{url=} not in DB {config.dbfile_twbm}")
        url = ""
    else:
        # (1, 'http://example.com', 'example title', ',tags1,', 'randomdesc', 0))
        bm_var = BukuDb(dbfile=config.dbfile_twbm).get_rec_by_id(id_)

        if "vimania" in bm_var[3]:
            _log.debug(f"Deleting twbm: {url}")
            if not BukuDb(dbfile=config.dbfile_twbm).delete_rec(
                index=id_, delay_commit=False
            ):
                raise VimaniaException(
                    f"Cannot delete {url=} from: {config.dbfile_twbm}"
                )
        else:
            _log.debug(f"{url=} not managed by vimania, no deletion.")
            url = "{url=} not managed by vimania, no deletion."

    return id_, url



import logging
import sys
from pathlib import Path
from typing import Optional

from pydantic import BaseSettings

_log = logging.getLogger("vimania-todos.environment")
ROOT_DIR = Path(__file__).parent.absolute()


class Environment(BaseSettings):
    log_level: str = "INFO"
    tw_vimania_db_url: str = f"sqlite:///{ROOT_DIR}/db/todos.db"
    twbm_db_url: Optional[str] = None  # = f"sqlite:///{ROOT_DIR}/db/bm.db"

    @property
    def dbfile(self):
        return f"{self.tw_vimania_db_url.split('sqlite:///')[-1]}"

    @property
    def dbfile_twbm(self):
        if self.twbm_db_url is None:
            return None
        return f"{self.twbm_db_url.split('sqlite:///')[-1]}"

    @property
    def is_installed_twbm(self):
        return self.twbm_db_url is not None


config = Environment()
_ = None



import logging
import re
from datetime import datetime
from typing import Sequence

from vimania_todos.db.dal import DAL, Todo, TodoStatus
from vimania_todos.environment import config
from vimania_todos.todos.handle_buffer import VimTodo

""" Implementation independent of vim """

_log = logging.getLogger("vimania-todos.core")


def create_todo_(args: str, path: str) -> int:
    todo = parse_todo_str(args)
    todo.path = path

    with DAL(env_config=config) as dal:
        todos = dal.get_todos(fts_query=todo.todo)
        _ = None
        if len(todos) >= 1:
            active_todos = [todo for todo in todos if todo.flags < TodoStatus.DONE]
            if len(active_todos) > 1:
                raise ValueError(
                    f"Same active todo already exists: {[td.id for td in active_todos]}. Clear DB inconsistency"
                )
            elif len(active_todos) == 1 and active_todos[0].id is not None:
                _log.debug(f"Updating todo: {todo.todo}")
                id_ = dal.update_todo(todo)
            else:
                _log.debug(f"Creating todo: {todo.todo}")
                id_ = dal.insert_todo(todo)
        return id_


def parse_todo_str(args: str) -> Todo:
    pattern = re.compile(r"""(- \[.+])(.*)""", re.MULTILINE)
    todo_status, todo = pattern.findall(args)[0]
    if todo_status == "- [ ]":
        flag = 0
    elif todo_status == "- [-]":
        flag = 1
    elif todo_status == "- [x]" or todo_status == "- [X]":
        flag = 2
    else:
        flag = 99  # unknown status
    todo = Todo(
        todo=todo.strip(),
        flags=flag,
        created_at=datetime.utcnow(),
    )
    return todo


def load_todos_() -> Sequence[str]:
    with DAL(env_config=config) as dal:
        todos = dal.get_todos(fts_query="")
        vtds = list()

        for todo in todos:
            vtd = VimTodo(
                raw_code=f"%%{todo.id}%%",
                todo=todo.todo,
            )
            vtd.set_status(todo.flags)
            vtds.append(vtd)
    return [vtd.vim_line for vtd in vtds]



#!/usr/bin/env python3
# encoding: utf-8

from enum import Enum


class JumpDirection(Enum):
    FORWARD = 1
    BACKWARD = 2


class Position:
    """Represents a Position in a text file: (0 based line index, 0 based column
    index) and provides methods for moving them around."""

    def __init__(self, line, col):
        self.line = line
        self.col = col

    def move(self, pivot, delta):
        """'pivot' is the position of the first changed character, 'delta' is
        how text after it moved."""
        if self < pivot:
            return
        if delta.line == 0:
            if self.line == pivot.line:
                self.col += delta.col
        elif delta.line > 0:
            if self.line == pivot.line:
                self.col += delta.col - pivot.col
            self.line += delta.line
        else:
            self.line += delta.line
            if self.line == pivot.line:
                self.col += -delta.col + pivot.col

    def delta(self, pos):
        """Returns the difference that the cursor must move to come from 'pos'
        to us."""
        assert isinstance(pos, Position)
        if self.line == pos.line:
            return Position(0, self.col - pos.col)
        if self > pos:
            return Position(self.line - pos.line, self.col)
        return Position(self.line - pos.line, pos.col)

    def __add__(self, pos):
        assert isinstance(pos, Position)
        return Position(self.line + pos.line, self.col + pos.col)

    def __sub__(self, pos):
        assert isinstance(pos, Position)
        return Position(self.line - pos.line, self.col - pos.col)

    def __eq__(self, other):
        return (self.line, self.col) == (other.line, other.col)

    def __ne__(self, other):
        return (self.line, self.col) != (other.line, other.col)

    def __lt__(self, other):
        return (self.line, self.col) < (other.line, other.col)

    def __le__(self, other):
        return (self.line, self.col) <= (other.line, other.col)

    def __repr__(self):
        return "(%i,%i)" % (self.line, self.col)

    def __getitem__(self, index):
        if index > 1:
            raise IndexError("position can be indexed only 0 (line) and 1 (column)")
        if index == 0:
            return self.line
        else:
            return self.col
